==> seven-elements-1.0.2%2B1.21.2-sources.jar: io/github/xrickastley/sevenelements/entity/DendroCoreEntity.java <==
package io.github.xrickastley.sevenelements.entity;

import com.mojang.serialization.Codec;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.UUID;

import org.jetbrains.annotations.Nullable;

import io.github.xrickastley.sevenelements.SevenElements;
import io.github.xrickastley.sevenelements.component.ElementComponent;
import io.github.xrickastley.sevenelements.element.Element;
import io.github.xrickastley.sevenelements.element.ElementalApplications;
import io.github.xrickastley.sevenelements.element.ElementalDamageSource;
import io.github.xrickastley.sevenelements.element.InternalCooldownContext;
import io.github.xrickastley.sevenelements.element.reaction.ElementalReaction;
import io.github.xrickastley.sevenelements.element.reaction.ElementalReactions;
import io.github.xrickastley.sevenelements.factory.SevenElementsSoundEvents;
import io.github.xrickastley.sevenelements.registry.SevenElementsDamageTypes;
import io.github.xrickastley.sevenelements.registry.SevenElementsEntityTypeTags;
import io.github.xrickastley.sevenelements.util.ClassInstanceUtil;
import io.github.xrickastley.sevenelements.util.Functions;
import io.github.xrickastley.sevenelements.util.JavaScriptUtil;

import net.fabricmc.fabric.api.networking.v1.PlayerLookup;
import net.fabricmc.fabric.api.networking.v1.ServerPlayNetworking;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1309;
import net.minecraft.class_1937;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_2487;
import net.minecraft.class_2499;
import net.minecraft.class_2512;
import net.minecraft.class_2520;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3419;
import net.minecraft.class_5699;
import net.minecraft.class_8710;
import net.minecraft.class_9129;
import net.minecraft.class_9135;
import net.minecraft.class_9139;

// Should technically extend Entity, but extends LivingEntity instead to NOT deal with more Networking and Spawn Packets.
public final class DendroCoreEntity extends SevenElementsEntity {
	private static final double SPRAWLING_SHOT_SPEED = 0.75;
	private static final double SPRAWLING_SHOT_GRAVITY = -0.05;
	private static final double SPRAWLING_SHOT_RADIUS = 24;
	private static final int SPRAWLING_SHOT_DELAY = 6;
	private static final double DENDRO_CORES_IN_RADIUS = 64;

	private List<UUID> owners;
	private @Nullable UUID target;
	private Type type = Type.NORMAL;
	private boolean exploded = false;
	private int hyperbloomAge = 0;
	private int curTicksInHitbox = 0;

	public DendroCoreEntity(class_1299<? extends class_1309> entityType, class_1937 world) {
		this(entityType, world, null);
	}

	public DendroCoreEntity(class_1299<? extends class_1309> entityType, class_1937 world, @Nullable class_1309 owner) {
		super(entityType, world);

		this.owners = new ArrayList<>();
		if (owner != null) this.owners.add(owner.method_5667());
	}

	public DendroCoreEntity setOwner(class_1309 owner) {
		this.owners = new ArrayList<>();

		if (owner != null) this.owners.add(owner.method_5667());

		return this;
	}

	public DendroCoreEntity addOwner(class_1309 owner) {
		if (owner != null) this.owners.add(owner.method_5667());

		return this;
	}

	public void setAsHyperbloom() {
		if (this.type != Type.NORMAL) throw new IllegalStateException("This DendroCoreEntity has already been transformed! Type: " + this.type);

		this.type = Type.HYPERBLOOM;
		this.hyperbloomAge = this.field_6012;
		this.field_5960 = true;
		this.method_5875(true);

		final @Nullable class_1309 target = ElementalReaction
			.getEntitiesInAoE(this, DendroCoreEntity.SPRAWLING_SHOT_RADIUS)
			.stream()
			.filter(e -> !(this.owners.contains(e.method_5667()) || e.method_29504() || e instanceof SevenElementsEntity || e.method_5864().method_20210(SevenElementsEntityTypeTags.IGNORED_TARGETS)))
			.min(Comparator.comparing(e -> e.method_5858(this)))
			.orElse(null);

		if (target == null) return;

		this.target = target.method_5667();
		this.sendSyncPayload();
	}

	public void setAsBurgeon() {
		if (this.type != Type.NORMAL) throw new IllegalStateException("This DendroCoreEntity has already been transformed! Type: " + this.type);

		this.type = Type.BURGEON;
		this.explode(3.0);
	}

	public boolean isNormal() {
		return this.type == Type.NORMAL;
	}

	public boolean isHyperbloom() {
		return this.type == Type.HYPERBLOOM;
	}

	public boolean isBurgeon() {
		return this.type == Type.BURGEON;
	}

	public void syncFromPayload(SyncDendroCoreS2CPayload payload) {
		this.type = payload.type;
		this.field_6012 = payload.age;
	}

	@Override
	public void method_5652(class_2487 nbt) {
		super.method_5652(nbt);

		nbt.method_10582("Type", this.type.toString());

		if (target != null) nbt.method_25927("Target", target);

		final class_2499 list = new class_2499();

		this.owners.forEach(
			Functions.consumer(Functions.compose(class_2512::method_25929, list::add))
		);

		nbt.method_10566("Owners", list);
	}

	@Override
	public void method_5749(class_2487 nbt) {
		super.method_5749(nbt);

		this.type = JavaScriptUtil.nullishCoalesing(
			nbt.method_10545("Type")
				? Type.valueOf(nbt.method_10558("Type"))
				: null,
			Type.NORMAL
		);

		this.target = nbt.method_10545("Target")
			? nbt.method_25926("Target")
			: null;

		this.owners.clear();

		nbt.method_10554("Owners", class_2520.field_33259)
			.forEach(
				Functions.consumer(Functions.compose(class_2512::method_25930, this.owners::add))
			);
	}

	private void doHyperbloom() {
		if (!(this.method_37908() instanceof final class_3218 world)) return;

		final int hyperbloomTick = this.field_6012 - this.hyperbloomAge;
		final class_1309 target = ClassInstanceUtil.castOrNull(world.method_14190(this.target), class_1309.class);

		if (target != null) {
			final class_243 targetPos = target.method_33571().method_1020(this.method_19538());
			final double distance = Math.sqrt(targetPos.field_1352 * targetPos.field_1352 + targetPos.field_1350 * targetPos.field_1350);
			final int ticks = Math.max(1, (int) (distance / DendroCoreEntity.SPRAWLING_SHOT_SPEED));

			// y value is derived from y(t) = y_0 + v_yt + \frac{1}{2}ay \times t^2
			final class_243 velocity = new class_243(
				targetPos.field_1352 / ticks,
				(targetPos.field_1351 - 0.5 * DendroCoreEntity.SPRAWLING_SHOT_GRAVITY * ticks * ticks) / ticks,
				targetPos.field_1350 / ticks
			);

			super.method_18799(velocity);

			final class_238 boundingBox = target.method_5829();

			if (!boundingBox.method_1006(this.method_19538())) return;

			this.curTicksInHitbox++;

			if (this.curTicksInHitbox < DendroCoreEntity.SPRAWLING_SHOT_DELAY) return;

			for (final class_1297 target2 : ElementalReaction.getEntitiesInAoE(target, 1.0, e -> !owners.contains(e.method_5667())))
				target2.method_64397(world, this.createDamageSource(target), ElementalReaction.getReactionDamage(this, 3.0));

			this.method_5650(class_5529.field_26998);

			this.method_37908()
				.method_8396(null, this.method_24515(), SevenElementsSoundEvents.SPRAWLING_SHOT_HIT, class_3419.field_15248, 0.5f, 1.0f);
		} else {
			super.method_18799(new class_243(0, 0.5, 0));

			if (hyperbloomTick >= 40) this.method_5650(class_5529.field_26998);
		}
	}

	@Override
	public void method_5768(class_3218 world) {
		this.explode(2.0);
	}

	@Override
	public boolean method_64397(class_3218 world, class_1282 source, float amount) {
		source = ElementComponent.applyElementalInfusions(source, this);

		if (!(source instanceof final ElementalDamageSource eds) || !this.isNormal()) return false;

		final Element element = eds.getElementalApplication().getElement();

		if (element != Element.PYRO && element != Element.ELECTRO) return false;

		final ElementalReaction reaction = element == Element.PYRO
			? ElementalReactions.BURGEON
			: ElementalReactions.HYPERBLOOM;

		reaction.trigger(this, ClassInstanceUtil.castOrNull(source.method_5529(), class_1309.class));

		return true;
	}

	@Override
	public void method_5773() {
		super.method_5773();

		if (this.field_6012 == 1) this.removeOldDendroCores();

		if (this.type == Type.HYPERBLOOM) this.doHyperbloom();

		if (this.field_6012 >= 120 && type != Type.HYPERBLOOM) {
			this.explode(2.0);
			this.method_5650(class_5529.field_26998);
		}
	}

	private void removeOldDendroCores() {
		if (!(this.method_37908() instanceof final class_3218 world)) return;

		final class_238 box = class_238.method_30048(this.method_30950(1f), DendroCoreEntity.DENDRO_CORES_IN_RADIUS, DendroCoreEntity.DENDRO_CORES_IN_RADIUS, DendroCoreEntity.DENDRO_CORES_IN_RADIUS);
		final List<DendroCoreEntity> dendroCores = this.method_37908().method_8390(DendroCoreEntity.class, box, dc -> true);

		if (dendroCores.size() <= 5) return;

		dendroCores.sort(Comparator.comparing(DendroCoreEntity::getAge).reversed());

		final Queue<DendroCoreEntity> queue = new LinkedList<>(dendroCores);

		while (queue.peek() != null && queue.size() > 5) queue.remove().method_5768(world);
	}

	private boolean explode(final double reactionMultiplier) {
		if (!(this.method_37908() instanceof final class_3218 world)) return false;

		if (this.exploded) return false;

		this.exploded = true;
		this.field_6012 = 117;

		if (!this.method_37908().field_9236) this.sendSyncPayload();

		final @Nullable class_1309 recentOwner = this.getRecentOwner();

		for (final class_1309 target : ElementalReaction.getEntitiesInAoE(this, 5.0)) {
			if (target instanceof DendroCoreEntity) continue;

			final ElementalDamageSource source = this.createDamageSource(target, recentOwner);

			float damage = ElementalReaction.getReactionDamage(this, reactionMultiplier);

			if (this.owners.contains(target.method_5667())) damage *= 0.02f;

			target.method_64397(world, source, damage);
		}

		this.method_37908()
			.method_8396(null, this.method_24515(), SevenElementsSoundEvents.DENDRO_CORE_EXPLOSION, class_3419.field_15248, 0.5f, 1.0f);

		return true;
	}

	private @Nullable class_1309 getRecentOwner() {
		return !owners.isEmpty() && this.method_37908() instanceof class_3218
			? this.getEntityFromUUID(owners.get(owners.size() - 1))
			: null;
	}

	private ElementalDamageSource createDamageSource(final class_1309 target) {
		final @Nullable class_1309 recentOwner = this.getRecentOwner();

		return this.createDamageSource(target, recentOwner);
	}

	private ElementalDamageSource createDamageSource(final class_1309 target, final class_1309 recentOwner) {
		return new ElementalDamageSource(
			this.method_37908()
				.method_48963()
				.method_48797(SevenElementsDamageTypes.DENDRO_CORE, this, recentOwner),
			ElementalApplications.gaugeUnits(target, Element.DENDRO, 0.0),
			InternalCooldownContext.ofNone(recentOwner)
		).shouldApplyDMGBonus(false);
	}

	private void sendSyncPayload() {
		final SyncDendroCoreS2CPayload packet = new SyncDendroCoreS2CPayload(this);

		for (final class_3222 otherPlayer : PlayerLookup.tracking(this))
			ServerPlayNetworking.send(otherPlayer, packet);
	}

	static {
		ElementComponent.denyElementsFor(DendroCoreEntity.class);
	}

	private static enum Type {
		NORMAL, HYPERBLOOM, BURGEON;

		static final Codec<Type> CODEC = class_5699.field_41759.xmap(Type::valueOf, Type::toString);
	}

	public static class SyncDendroCoreS2CPayload implements class_8710 {
		public static final class_8710.class_9154<SyncDendroCoreS2CPayload> ID = new class_8710.class_9154<>(
			SevenElements.identifier("s2c/sync_dendro_core")
		);

		public static final class_9139<class_9129, SyncDendroCoreS2CPayload> CODEC = class_9139.method_56436(
			class_9135.field_49675, inst -> inst.entityId,
			class_9135.field_49675, inst -> inst.age,
			class_9135.method_56368(DendroCoreEntity.Type.CODEC), inst -> inst.type,
			SyncDendroCoreS2CPayload::new
		);

		private final int entityId;
		private final int age;
		private final DendroCoreEntity.Type type;

		private SyncDendroCoreS2CPayload(int entityId, int age, DendroCoreEntity.Type type) {
			this.entityId = entityId;
			this.age = age;
			this.type = type;
		}

		private SyncDendroCoreS2CPayload(final DendroCoreEntity dendroCore) {
			this.entityId = dendroCore.method_5628();
			this.age = dendroCore.getAge();
			this.type = dendroCore.type;
		}

		@Override
		public class_9154<? extends class_8710> method_56479() {
			return ID;
		}

		public int entityId() {
			return this.entityId;
		}
	}
}

==> seven-elements-1.0.3%2B1.21.5-sources.jar: io/github/xrickastley/sevenelements/entity/DendroCoreEntity.java <==
package io.github.xrickastley.sevenelements.entity;

import com.mojang.serialization.Codec;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.UUID;

import org.jetbrains.annotations.Nullable;

import io.github.xrickastley.sevenelements.SevenElements;
import io.github.xrickastley.sevenelements.component.ElementComponent;
import io.github.xrickastley.sevenelements.element.Element;
import io.github.xrickastley.sevenelements.element.ElementalApplications;
import io.github.xrickastley.sevenelements.element.ElementalDamageSource;
import io.github.xrickastley.sevenelements.element.InternalCooldownContext;
import io.github.xrickastley.sevenelements.element.reaction.ElementalReaction;
import io.github.xrickastley.sevenelements.element.reaction.ElementalReactions;
import io.github.xrickastley.sevenelements.factory.SevenElementsSoundEvents;
import io.github.xrickastley.sevenelements.registry.SevenElementsDamageTypes;
import io.github.xrickastley.sevenelements.registry.SevenElementsEntityTypeTags;
import io.github.xrickastley.sevenelements.util.ClassInstanceUtil;
import io.github.xrickastley.sevenelements.util.NbtHelper;

import net.fabricmc.fabric.api.networking.v1.PlayerLookup;
import net.fabricmc.fabric.api.networking.v1.ServerPlayNetworking;
import net.minecraft.class_1282;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1309;
import net.minecraft.class_1937;
import net.minecraft.class_238;
import net.minecraft.class_243;
import net.minecraft.class_2487;
import net.minecraft.class_3218;
import net.minecraft.class_3222;
import net.minecraft.class_3419;
import net.minecraft.class_4844;
import net.minecraft.class_5699;
import net.minecraft.class_8710;
import net.minecraft.class_9129;
import net.minecraft.class_9135;
import net.minecraft.class_9139;

// Should technically extend Entity, but extends LivingEntity instead to NOT deal with more Networking and Spawn Packets.
public final class DendroCoreEntity extends SevenElementsEntity {
	private static final double SPRAWLING_SHOT_SPEED = 0.75;
	private static final double SPRAWLING_SHOT_GRAVITY = -0.05;
	private static final double SPRAWLING_SHOT_RADIUS = 24;
	private static final int SPRAWLING_SHOT_DELAY = 6;
	private static final double DENDRO_CORES_IN_RADIUS = 64;

	private List<UUID> owners;
	private @Nullable UUID target;
	private Type type = Type.NORMAL;
	private boolean exploded = false;
	private int hyperbloomAge = 0;
	private int curTicksInHitbox = 0;

	public DendroCoreEntity(class_1299<? extends class_1309> entityType, class_1937 world) {
		this(entityType, world, null);
	}

	public DendroCoreEntity(class_1299<? extends class_1309> entityType, class_1937 world, @Nullable class_1309 owner) {
		super(entityType, world);

		this.owners = new ArrayList<>();
		if (owner != null) this.owners.add(owner.method_5667());
	}

	public DendroCoreEntity setOwner(class_1309 owner) {
		this.owners = new ArrayList<>();

		if (owner != null) this.owners.add(owner.method_5667());

		return this;
	}

	public DendroCoreEntity addOwner(class_1309 owner) {
		if (owner != null) this.owners.add(owner.method_5667());

		return this;
	}

	public void setAsHyperbloom() {
		if (this.type != Type.NORMAL) throw new IllegalStateException("This DendroCoreEntity has already been transformed! Type: " + this.type);

		this.type = Type.HYPERBLOOM;
		this.hyperbloomAge = this.field_6012;
		this.field_5960 = true;
		this.method_5875(true);

		final @Nullable class_1309 target = ElementalReaction
			.getEntitiesInAoE(this, DendroCoreEntity.SPRAWLING_SHOT_RADIUS)
			.stream()
			.filter(e -> !(this.owners.contains(e.method_5667()) || e.method_29504() || e instanceof SevenElementsEntity || e.method_5864().method_20210(SevenElementsEntityTypeTags.IGNORED_TARGETS)))
			.min(Comparator.comparing(e -> e.method_5858(this)))
			.orElse(null);

		if (target == null) return;

		this.target = target.method_5667();
		this.sendSyncPayload();
	}

	public void setAsBurgeon() {
		if (this.type != Type.NORMAL) throw new IllegalStateException("This DendroCoreEntity has already been transformed! Type: " + this.type);

		this.type = Type.BURGEON;
		this.explode(3.0);
	}

	public boolean isNormal() {
		return this.type == Type.NORMAL;
	}

	public boolean isHyperbloom() {
		return this.type == Type.HYPERBLOOM;
	}

	public boolean isBurgeon() {
		return this.type == Type.BURGEON;
	}

	public void syncFromPayload(SyncDendroCoreS2CPayload payload) {
		this.type = payload.type;
		this.field_6012 = payload.age;
	}

	@Override
	public void method_5652(class_2487 nbt) {
		super.method_5652(nbt);

		nbt.method_67494("Type", DendroCoreEntity.Type.CODEC, this.type);
		nbt.method_68082("Target", class_4844.field_40825, target);

		NbtHelper.putList(nbt, "Owners", class_4844.field_40825, this.owners);
	}

	@Override
	public void method_5749(class_2487 nbt) {
		super.method_5749(nbt);

		this.type = nbt.method_67491("Type", DendroCoreEntity.Type.CODEC).orElse(Type.NORMAL);
		this.target = nbt.method_67491("Target", class_4844.field_40825).orElse(null);

		this.owners.clear();
		this.owners.addAll(NbtHelper.getList(nbt, "Owners", class_4844.field_40825));
	}

	private void doHyperbloom() {
		if (!(this.method_37908() instanceof final class_3218 world)) return;

		final int hyperbloomTick = this.field_6012 - this.hyperbloomAge;
		final class_1309 target = ClassInstanceUtil.castOrNull(world.method_66347(this.target), class_1309.class);

		if (target != null) {
			final class_243 targetPos = target.method_33571().method_1020(this.method_19538());
			final double distance = Math.sqrt(targetPos.field_1352 * targetPos.field_1352 + targetPos.field_1350 * targetPos.field_1350);
			final int ticks = Math.max(1, (int) (distance / DendroCoreEntity.SPRAWLING_SHOT_SPEED));

			// y value is derived from y(t) = y_0 + v_yt + \frac{1}{2}ay \times t^2
			final class_243 velocity = new class_243(
				targetPos.field_1352 / ticks,
				(targetPos.field_1351 - 0.5 * DendroCoreEntity.SPRAWLING_SHOT_GRAVITY * ticks * ticks) / ticks,
				targetPos.field_1350 / ticks
			);

			super.method_18799(velocity);

			final class_238 boundingBox = target.method_5829();

			if (!boundingBox.method_1006(this.method_19538())) return;

			this.curTicksInHitbox++;

			if (this.curTicksInHitbox < DendroCoreEntity.SPRAWLING_SHOT_DELAY) return;

			for (final class_1297 target2 : ElementalReaction.getEntitiesInAoE(target, 1.0, e -> !owners.contains(e.method_5667())))
				target2.method_64397(world, this.createDamageSource(target), ElementalReaction.getReactionDamage(this, 3.0));

			this.method_5650(class_5529.field_26998);

			this.method_37908()
				.method_8396(null, this.method_24515(), SevenElementsSoundEvents.SPRAWLING_SHOT_HIT, class_3419.field_15248, 0.5f, 1.0f);
		} else {
			super.method_18799(new class_243(0, 0.5, 0));

			if (hyperbloomTick >= 40) this.method_5650(class_5529.field_26998);
		}
	}

	@Override
	public void method_5768(class_3218 world) {
		this.explode(2.0);
	}

	@Override
	public boolean method_64397(class_3218 world, class_1282 source, float amount) {
		source = ElementComponent.applyElementalInfusions(source, this);

		if (!(source instanceof final ElementalDamageSource eds) || !this.isNormal()) return false;

		final Element element = eds.getElementalApplication().getElement();

		if (element != Element.PYRO && element != Element.ELECTRO) return false;

		final ElementalReaction reaction = element == Element.PYRO
			? ElementalReactions.BURGEON
			: ElementalReactions.HYPERBLOOM;

		reaction.trigger(this, ClassInstanceUtil.castOrNull(source.method_5529(), class_1309.class));

		return true;
	}

	@Override
	public void method_5773() {
		super.method_5773();

		if (this.field_6012 == 1) this.removeOldDendroCores();

		if (this.type == Type.HYPERBLOOM) this.doHyperbloom();

		if (this.field_6012 >= 120 && type != Type.HYPERBLOOM) {
			this.explode(2.0);
			this.method_5650(class_5529.field_26998);
		}
	}

	private void removeOldDendroCores() {
		if (!(this.method_37908() instanceof final class_3218 world)) return;

		final class_238 box = class_238.method_30048(this.method_30950(1f), DendroCoreEntity.DENDRO_CORES_IN_RADIUS, DendroCoreEntity.DENDRO_CORES_IN_RADIUS, DendroCoreEntity.DENDRO_CORES_IN_RADIUS);
		final List<DendroCoreEntity> dendroCores = this.method_37908().method_8390(DendroCoreEntity.class, box, dc -> true);

		if (dendroCores.size() <= 5) return;

		dendroCores.sort(Comparator.comparing(DendroCoreEntity::getAge).reversed());

		final Queue<DendroCoreEntity> queue = new LinkedList<>(dendroCores);

		while (queue.peek() != null && queue.size() > 5) queue.remove().method_5768(world);
	}

	private boolean explode(final double reactionMultiplier) {
		if (!(this.method_37908() instanceof final class_3218 world)) return false;

		if (this.exploded) return false;

		this.exploded = true;
		this.field_6012 = 117;

		if (!this.method_37908().field_9236) this.sendSyncPayload();

		final @Nullable class_1309 recentOwner = this.getRecentOwner();

		for (final class_1309 target : ElementalReaction.getEntitiesInAoE(this, 5.0)) {
			if (target instanceof DendroCoreEntity) continue;

			final ElementalDamageSource source = this.createDamageSource(target, recentOwner);

			float damage = ElementalReaction.getReactionDamage(this, reactionMultiplier);

			if (this.owners.contains(target.method_5667())) damage *= 0.02f;

			target.method_64397(world, source, damage);
		}

		this.method_37908()
			.method_8396(null, this.method_24515(), SevenElementsSoundEvents.DENDRO_CORE_EXPLOSION, class_3419.field_15248, 0.5f, 1.0f);

		return true;
	}

	private @Nullable class_1309 getRecentOwner() {
		return !owners.isEmpty() && this.method_37908() instanceof class_3218
			? this.getEntityFromUUID(owners.get(owners.size() - 1))
			: null;
	}

	private ElementalDamageSource createDamageSource(final class_1309 target) {
		final @Nullable class_1309 recentOwner = this.getRecentOwner();

		return this.createDamageSource(target, recentOwner);
	}

	private ElementalDamageSource createDamageSource(final class_1309 target, final class_1309 recentOwner) {
		return new ElementalDamageSource(
			this.method_37908()
				.method_48963()
				.method_48797(SevenElementsDamageTypes.DENDRO_CORE, this, recentOwner),
			ElementalApplications.gaugeUnits(target, Element.DENDRO, 0.0),
			InternalCooldownContext.ofNone(recentOwner)
		).shouldApplyDMGBonus(false);
	}

	private void sendSyncPayload() {
		final SyncDendroCoreS2CPayload packet = new SyncDendroCoreS2CPayload(this);

		for (final class_3222 otherPlayer : PlayerLookup.tracking(this))
			ServerPlayNetworking.send(otherPlayer, packet);
	}

	static {
		ElementComponent.denyElementsFor(DendroCoreEntity.class);
	}

	private static enum Type {
		NORMAL, HYPERBLOOM, BURGEON;

		static final Codec<Type> CODEC = class_5699.field_41759.xmap(Type::valueOf, Type::toString);
	}

	public static class SyncDendroCoreS2CPayload implements class_8710 {
		public static final class_8710.class_9154<SyncDendroCoreS2CPayload> ID = new class_8710.class_9154<>(
			SevenElements.identifier("s2c/sync_dendro_core")
		);

		public static final class_9139<class_9129, SyncDendroCoreS2CPayload> CODEC = class_9139.method_56436(
			class_9135.field_49675, inst -> inst.entityId,
			class_9135.field_49675, inst -> inst.age,
			class_9135.method_56368(DendroCoreEntity.Type.CODEC), inst -> inst.type,
			SyncDendroCoreS2CPayload::new
		);

		private final int entityId;
		private final int age;
		private final DendroCoreEntity.Type type;

		private SyncDendroCoreS2CPayload(int entityId, int age, DendroCoreEntity.Type type) {
			this.entityId = entityId;
			this.age = age;
			this.type = type;
		}

		private SyncDendroCoreS2CPayload(final DendroCoreEntity dendroCore) {
			this.entityId = dendroCore.method_5628();
			this.age = dendroCore.getAge();
			this.type = dendroCore.type;
		}

		@Override
		public class_9154<? extends class_8710> method_56479() {
			return ID;
		}

		public int entityId() {
			return this.entityId;
		}
	}
}
